--[[
	Addon: EnergyCapped
	Version: r2
	Author: sp00n

	Mod to see if and when you capped energy during an encounter.
	Also displays a summary at the end of the fight.
	You can add a new chat channel named "EnergyCapped" to have a more clear overview
	It currently only activates for bosses, according to LibBossIDs-1.0

	
	TODO:
	- Clean up code. Too many variables to be checked for.
	- Add slash commands for print output, channel name, solo/party/raid maybe?
--]]

local active     = false	-- Is the addon currently active? Triggers when engaging a boss present in BossIDs.
local maxEnergy  = 100		-- Your max energy level
local inCombat   = false	-- Are you in combat
local hasEngaged = false	-- Have you actually hit the target yet
local isCapped   = false	-- Are you energy capped
local capCount   = 0		-- How often have you capped
local wasted     = {		-- Wasted energy
	Regen = 0,					-- From regeneration while being capped
	Procs = 0					-- From procs while being capped
}
local duration   = 0		-- Total duration of being capped
local playerGUID = ""		-- Your GUID
local regen      = 0		-- Your energy regeneration rate
local startTime  = 0		-- Starting time of a capped period

local maxRegen   = 0		-- Maximum regeneration value during the fight
local avgRegen   = 0		-- Average regeneration value during the fight
local cbtDur     = 0		-- Duration of the combat
local cbtStart   = 0		-- Start time of combat
local startTime2 = 0		-- Start time of the last regenartion value

local useCustChan = false	-- Is there a custom chat channel named "CappedEnergy" present?
local chanNum     = 0		-- If yes, which chat number does it have (ChatFrameX)


-- Change that to false if you don't want print output
local doPrint     = true	-- Should it do a normal print as well even if a custom chat window is found (no slash command yet)?


-- Caching
local UnitPower     = UnitPower
local GetPowerRegen = GetPowerRegen
local GetTime       = GetTime
local floor         = math.floor
local UnitClassification = UnitClassification


EnergyCapped = CreateFrame("Frame", "EnergyCapped", UIParent)
local addon  = EnergyCapped

-- Register the events to the addon
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, event, ...)
end)

-- Get boss IDs
addon.BossIDs  = LibStub("LibBossIDs-1.0").BossIDs
addon.ExtraIDs = {	-- Extra IDs, not inlcuded in LibBossIDs
	31146,	-- Raider's Training Dummy
	52577,	-- Rhyolith Left Foot
	53087,	-- Rhyolith Right Foot
	53694,	-- Riplimb
	53695,	-- Rageface
}

addon:RegisterEvent("ADDON_LOADED")


-- Helper function
function round(num, length)
	local mult = 10^(length or 2)
	return floor(num * mult + 0.5) / mult	
end


-- Loading addon
function addon.ADDON_LOADED(self,event,arg1)
    if arg1 ~= "EnergyCapped" then return end
    local class = select(2, UnitClass("player"))
    if (class ~= "ROGUE" and class ~= "DRUID") then return end
    
    self:RegisterEvent("UNIT_MAXPOWER")				-- Maximum energy level
    self:RegisterEvent("UNIT_POWER")				-- Fired when current energy level changes
    self:RegisterEvent("PLAYER_REGEN_DISABLED")		-- Entering combat
    self:RegisterEvent("PLAYER_REGEN_ENABLED")		-- Leaving combat
    self:RegisterEvent("UNIT_DISPLAYPOWER")			-- Fired when something in the energy max level changes (Respec, etc.)
    self:RegisterEvent("LFG_LOCK_INFO_RECEIVED")	-- Instead of "PLAYER_ENTERING_WORLD"
    
    
    --self:RegisterEvent("UNIT_ATTACK_SPEED")		-- Fires when melee haste changes. Possibly also for SnD, so disabled
    self:RegisterEvent("PLAYER_DAMAGE_DONE_MODS")	-- Fires when energy regeneration rate changes. Does NOT fire for normal haste effects
    self:RegisterEvent("UNIT_SPELL_HASTE")			-- Fires for normal haste effects only. Hopefully.
    self.UNIT_SPELL_HASTE = self.PLAYER_DAMAGE_DONE_MODS
    
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")	-- Combat log event, checking for energy procs (SPELL_ENERGIZE, SPELL_PERIODIC_ENERGIZE)
    
    -- Check if "EnergyCapped" chat frame is present
    self:CheckForChatChannel()
end


-- Check if "EnergyCapped" chat frame is present
function addon:CheckForChatChannel()
	for i = 1, NUM_CHAT_WINDOWS do
		local cf = _G["ChatFrame"..i.."Tab"]
		if (cf:GetText() == "EnergyCapped") then
			chanNum = i
			useCustChannel = true
			return
		end
	end
end


-- Log in
function addon.LFG_LOCK_INFO_RECEIVED(self)
	playerGUID = UnitGUID("player")
	self:UNIT_DISPLAYPOWER()
end


-- Something changed in the max energy level
function addon.UNIT_DISPLAYPOWER(self)
    self:UNIT_MAXPOWER()
    regen = select(1, GetPowerRegen())
end


-- Get Max energy
function addon.UNIT_MAXPOWER(self)
    maxEnergy = UnitPowerMax("player")
end


-- Energy changes
function addon.UNIT_POWER(self, event, unit, powertype)
    if (not active) then return end
    if (not inCombat) then return end
    if (not hasEngaged) then return end

    local power = UnitPower("player")
    
    -- Energy capped for the first time
    if (isCapped == false and power == maxEnergy) then
    	startTime = GetTime()
    	regen     = select(1, GetPowerRegen())
    	isCapped  = true
    	capCount  = capCount + 1

    	self:Print("You just capped energy! ("..power.."/"..maxEnergy.." with "..round(regen).." regen)")
    
    -- Still capped, but not performing anything
    --elseif (isCapped == true and power == maxEnergy) then
		-- nothing
        
    -- Not capped anymore
    elseif (isCapped == true and power < maxEnergy) then
    	local curDur = GetTime() - startTime
    	duration  = duration + curDur
    	startTime = 0
    	regen     = select(1, GetPowerRegen())
		wasted.Regen = wasted.Regen + (regen * curDur)					-- Add normal energy regeneration to wasted energy
    	isCapped  = false
    end
end


-- Fires when energy regeneration rate changes.
-- Called for PLAYER_DAMAGE_DONE_MODS and UNIT_SPELL_HASTE
function addon.PLAYER_DAMAGE_DONE_MODS(self, event, unitID)
	if (not active) then return end
	if (not inCombat) then return end
	
	local now, oldDur, curRegen
	now      = GetTime()
	curRegen = select(1, GetPowerRegen())	-- New regeneration value after the buff applied/faded
	
	
	-- Only if regeneration value really changed
	if (curRegen ~= regen) then
		--self:Print("Energy regen changed! ("..regen.." -> "..curRegen.."), "..event)
		
		-- Duration of last regeneration value
		oldDur     = now - startTime2	-- Duration of last regeneration value
	
		-- Regeneration values
		maxRegen   = math.max(maxRegen, curRegen)	

		--self:Print("("..avgRegen.." * "..cbtDur.." + "..regen.." * "..oldDur..") / ("..cbtDur.." + "..oldDur..")")
		--self:Print("("..avgRegen * cbtDur.." + "..regen * oldDur..") / ("..cbtDur + oldDur..")")
		--self:Print("("..avgRegen * cbtDur + regen * oldDur..") / ("..cbtDur + oldDur..")")
		--self:Print((avgRegen * cbtDur + regen * oldDur) / (cbtDur + oldDur))
		
		avgRegen   = (avgRegen * cbtDur + regen * oldDur) / (cbtDur + oldDur)	-- New average regeneration value
		cbtDur     = cbtDur + oldDur	-- Total combat duration so far
		
		-- Restart the clock for the regeneration values
		startTime2 = now
	end
	
	
	-- Proceed with calculations for when being capped
	if (isCapped) then
		-- Duration of being capped with the old regeneration value
		oldDur   = now - startTime		-- Duration of last regeneration value
		duration = duration + oldDur	-- Total duration of being capped
		
		-- Wasted value with old regeneration rate
		wasted.Regen = wasted.Regen + (oldDur * regen)
		
		-- Restart the clock
		startTime = now
	end

	-- Set new regeneration value for next calculation
	regen = curRegen
end



-- Check combat log events
-- See if player hit the target to set hasEngaged
-- Power gains through procs
function addon.COMBAT_LOG_EVENT_UNFILTERED(self, evnt1, timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceFlags2, destGUID, destName, destFlags, destFlags2, ...)
	if (not inCombat) then return end
	
	-- Start the calculation when you first hit the boss
	if (not active and not hasEngaged and sourceGUID == playerGUID and (event == "SPELL_DAMAGE" or event == "SPELL_MISSED" or event == "SWING_DAMAGE" or event == "SWING_MISSED" )) then
		
		local guid = tonumber(destGUID:sub(-13, -9), 16)
		guid = guid or -1
		
		-- Is target a boss?
		if (UnitClassification("target") == "worldboss" or addon.BossIDs[guid] or addon.ExtraIDs[guid]) then
			active       = true
			hasEngaged   = true
			startTime2   = GetTime()
			cbtStart     = startTime2
			regen        = select(1, GetPowerRegen())
			maxRegen     = regen
			avgRegen     = regen
			
			self:Print("Target Engaged: "..destName.." ("..round(regen).." regen)")
		end
	end

	-- Proceed with capped calculation
	if (not active) then return end
	if (not isCapped) then return end	
	if (not hasEngaged) then return end
	if (destGUID ~= playerGUID) then return end
	if (event ~= "SPELL_ENERGIZE" and event ~= "SPELL_PERIODIC_ENERGIZE") then return end
	
	-- Energy procs
	spellId, spellName, spellSchool, amount, powerType = select(1, ...)
		
	-- Wasted energy due to procs
	if (powerType == 3) then
		wasted.Procs = wasted.Procs + amount
	end
end


-- Combat starts
function addon.PLAYER_REGEN_DISABLED(self)
	-- Reset variables
	active       = false
	inCombat     = true
	isCapped     = false
	hasEngaged   = false
	capCount     = 0
	wasted.Procs = 0
	wasted.Regen = 0
	duration     = 0
	startTime    = 0
	
	-- Regeneration values
	regen        = 0
	maxRegen     = 0
	avgRegen     = 0
	cbtDur       = 0
	startTime2   = 0
	cbtStart     = 0

	--self:Print("Combat started. Time Code: "..startTime2)
	self:CheckForChatChannel()
end


-- Combat has ended
-- Display summary
function addon.PLAYER_REGEN_ENABLED(self)
	if (not active) then return end
	
	local curRegen, now, oldDur, curDur, text
	text = {}
	table.insert(text, "|cff999999-----------|r |cff1C97FBENERGY CAPPED REPORT|r |cff999999-----------|r")
	
	-- Get regen value
	curRegen = select(1, GetPowerRegen())	-- New regeneration value after the buff applied/faded
	
	-- Duration of last regeneration value
	now        = GetTime()
	oldDur     = now - startTime	-- Duration of last regeneration value
	
	-- Regeneration values
	maxRegen   = math.max(maxRegen, curRegen)	
	avgRegen   = (avgRegen * cbtDur + regen * oldDur) / (cbtDur + oldDur)	-- New average regeneration value
	cbtDur     = cbtDur + oldDur	-- Total combat duration so far
	
	table.insert(text, "|cff999999<>|r Combat ended after |cff1C97FB"..round(now-cbtStart).."|r seconds.")
	
	-- At least capped once
	if (capCount > 0) then
		-- Energy capped when combat ended
		if (isCapped == true) then
			local curDur = GetTime() - startTime				-- Duration of last capped timeframe
			
			duration  = duration + curDur						-- Total duration of being capped
	    	regen = select(1, GetPowerRegen())					-- Current regeneration rate
			wasted.Regen = wasted.Regen + (regen * curDur)		-- Add normal energy regeneration to wasted energy
		end
		
		table.insert(text, "|cff999999<>|r You capped energy |cffff0000"..capCount.." times|r in the last fight!")
		table.insert(text, "|cff999999<>|r The total duration was |cffff0000"..round(duration).."s|r.")
		table.insert(text, "|cff999999<>|r You wasted |cffff0000~"..round(wasted.Procs+wasted.Regen).." energy|r during that time.")
		table.insert(text, "|cff999999<>|r (~"..round(wasted.Regen).." from Regeneration / "..round(wasted.Procs).." from Procs)")
		
	
	-- Not wasted any energy!
	else
		table.insert(text, "|cff999999<>|r You didn't waste any energy. Good job!")
	end
	
	table.insert(text, "|cff999999<>|r Your maximum energy regeneration rate was |cff1C97FB"..round(maxRegen).."|r.")
	table.insert(text, "|cff999999<>|r Your average energy regeneration rate was |cff1C97FB"..round(avgRegen).."|r.")
	table.insert(text, "|cff999999-----------|r |cff1C97FBENERGY CAPPED REPORT|r |cff999999-----------|r")
	
	for k,v in ipairs(text) do self:Print(v) end
	
	
	-- Reset variables
	active       = false
	inCombat     = false
	isCapped     = false
	hasEngaged   = false
	capCount     = 0
	wasted.Procs = 0
	wasted.Regen = 0
	duration     = 0
	startTime    = 0
	
	-- Regeneration values
	maxRegen     = 0
	avgRegen     = 0
	cbtStart     = 0
	cbtDur       = 0
	startTime2   = 0
end


-- Only print the text or also display in custom channel?
function addon:Print(text)
	if (useCustChannel and chanNum > 0) then
		_G["ChatFrame"..chanNum]:AddMessage(text)
	end
	
	if (doPrint) then print(text) end
end